﻿/* Name: main.c
 * Project: hid-mouse, a very simple HID example
 * Author: Christian Starkjohann
 * Creation Date: 2008-04-07
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 * This Revision: $Id$
 */

/*
This example should run on most AVRs with only little changes. No special
hardware resources except INT0 are used. You may have to change usbconfig.h for
different I/O pins for USB. Please note that USB D+ must be the INT0 pin, or
at least be connected to INT0 as well.

We use VID/PID 0x046D/0xC00E which is taken from a Logitech mouse. Don't
publish any hardware using these IDs! This is for demonstration only!
*/
//#define F_CPU 12000000UL
#include <avr/io.h>
//#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */

//#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
//#include "oddebug.h"        /* This is also an example for using debug macros */


/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

PROGMEM const char usbHidReportDescriptor[119] = { /* USB report descriptor, size must match usbconfig.h */
	
		    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
		    0x09, 0x06,                    // USAGE (Keyboard)
		    0xa1, 0x01,                    // COLLECTION (Application)
		    0x85, 0x01,                    //   REPORT_ID (1)
		    0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
		    0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
		    0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
		    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
		    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
		    0x75, 0x01,                    //   REPORT_SIZE (1)
		    0x95, 0x08,                    //   REPORT_COUNT (8)
		    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
		    0x95, 0x01,                    //   REPORT_COUNT (1)
		    0x75, 0x08,                    //   REPORT_SIZE (8)
		    0x81, 0x03,                    //   INPUT (Cnst,Var,Abs)
		    0x95, 0x05,                    //   REPORT_COUNT (5)
		    0x75, 0x01,                    //   REPORT_SIZE (1)
		    0x05, 0x08,                    //   USAGE_PAGE (LEDs)
		    0x19, 0x01,                    //   USAGE_MINIMUM (Num Lock)
		    0x29, 0x05,                    //   USAGE_MAXIMUM (Kana)
		    0x91, 0x02,                    //   OUTPUT (Data,Var,Abs)
		    0x95, 0x01,                    //   REPORT_COUNT (1)
		    0x75, 0x03,                    //   REPORT_SIZE (3)
		    0x91, 0x03,                    //   OUTPUT (Cnst,Var,Abs)
		    0x95, 0x05,                    //   REPORT_COUNT (5)
		    0x75, 0x08,                    //   REPORT_SIZE (8)
		    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
		    0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
		    0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
		    0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
		    0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
		    0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
		    0xc0,                           // END_COLLECTION
			
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x02,                    // USAGE (Mouse)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x09, 0x01,                    //   USAGE (Pointer)
    0xA1, 0x00,                    //   COLLECTION (Physical)
	0x85, 0x02,                    //     REPORT_ID (2)
    0x05, 0x09,                    //     USAGE_PAGE (Button)
    0x19, 0x01,                    //     USAGE_MINIMUM
    0x29, 0x03,                    //     USAGE_MAXIMUM
    0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //     LOGICAL_MAXIMUM (1)
    0x95, 0x03,                    //     REPORT_COUNT (3)
    0x75, 0x01,                    //     REPORT_SIZE (1)
    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
    0x95, 0x01,                    //     REPORT_COUNT (1)
    0x75, 0x05,                    //     REPORT_SIZE (5)
    0x81, 0x03,                    //     INPUT (Const,Var,Abs)
    0x05, 0x01,                    //     USAGE_PAGE (Generic Desktop)
    0x09, 0x30,                    //     USAGE (X)
    0x09, 0x31,                    //     USAGE (Y)
    0x09, 0x38,                    //     USAGE (Wheel)
    0x15, 0x81,                    //     LOGICAL_MINIMUM (-127)
    0x25, 0x7F,                    //     LOGICAL_MAXIMUM (127)
    0x75, 0x08,                    //     REPORT_SIZE (8)
    0x95, 0x03,                    //     REPORT_COUNT (3)
    0x81, 0x06,                    //     INPUT (Data,Var,Rel)
    0xC0,                          //   END_COLLECTION
    0xC0,                          // END COLLECTION
};


/* This is the same report descriptor as seen in a Logitech mouse. The data
 * described by this descriptor consists of 4 bytes:
 *      .  .  .  .  . B2 B1 B0 .... one byte with mouse button states
 *     X7 X6 X5 X4 X3 X2 X1 X0 .... 8 bit signed relative coordinate x
 *     Y7 Y6 Y5 Y4 Y3 Y2 Y1 Y0 .... 8 bit signed relative coordinate y
 *     W7 W6 W5 W4 W3 W2 W1 W0 .... 8 bit signed relative coordinate wheel
 */
typedef struct {
	uint8_t id;
	uint8_t modifier;
	uint8_t reserved;
	uint8_t keycode[5];
} keyboard_report_t;

typedef struct{
	uchar   id;
    uchar   buttonMask;
    char    dx;
    char    dy;
    char    dWheel;
}report_t;

#define MOD_CONTROL_LEFT    (1<<0)
#define MOD_SHIFT_LEFT      (1<<1)
#define MOD_ALT_LEFT        (1<<2)
#define MOD_GUI_LEFT        (1<<3)
#define MOD_CONTROL_RIGHT   (1<<4)
#define MOD_SHIFT_RIGHT     (1<<5)
#define MOD_ALT_RIGHT       (1<<6)
#define MOD_GUI_RIGHT       (1<<7)

static report_t reportBuffer;
static keyboard_report_t kbdBuffer;
static int      sinus = 7 << 6, cosinus = 0;
static uchar    idleRate;   /* repeat rate for keyboards, never used for mice */
volatile uchar	LED_state;
uchar	LED_state_save;

uchar buffKey[4]={0x62,0x5e,0x5d,0x00};
uchar buffstate=0;
uchar bufflock=0;

uchar buffText[254]="DTViMS - Super and Nastya - beloved wife";
uchar buffTextLen=40;
uchar buffTextState=0;
int testpoll=0;

/*
unsigned char read_from_internal_eeprom(unsigned int addr)
{
	while (EECR & (1<<EEWE)); //wait for write to finish
	EEAR = addr; //load the address to read from
	EECR |= (1<<EERE);//set the read flag
	//the clock is stopped for 4 cyles
	EEAR =0; //leave address at 0 so if power down corrupts location 0
	return EEDR;
}

unsigned char write_internal_eeprom(unsigned int addr,char data)
{
	char cnt;
	for(cnt=0;cnt<10;cnt++)
	{
		if(data==(read_from_internal_eeprom(addr))) //will check in the read routine for write flag being cleared
		break;
		EEAR = addr;//load up address
		EEDR = data; //load up data
		//cli();//disable interrupts
		EECR |= (1<<EEMWE); //enable write
		EECR |= (1<<EEWE); //start write
		//sei(); //enable interrupts
	}
	return cnt;
}
*/

static void buildkey(){
	if(bufflock){
		//kbdBuffer.modifier=MOD_ALT_RIGHT;
		kbdBuffer.keycode[0]=0;
		bufflock=0;
		if(buffstate>3){
			buffstate=0;
		}
	}else{
		if(buffstate<3){
			kbdBuffer.modifier=MOD_ALT_RIGHT;
		}else{
			kbdBuffer.modifier=0;
		}
		
		kbdBuffer.keycode[0]=buffKey[buffstate];
		buffstate++;
		bufflock=1;
	}	
}

static uchar kbdPrintText(){
	uchar ch;
	uchar i;
	if(buffTextState>=buffTextLen && buffstate==0){
		_delay_ms(1);

		if(!(LED_state_save & 0b00000001) && LED_state & 0b00000001){
			LED_state_save |= 0b00000001;
			kbdBuffer.modifier = 0;
			kbdBuffer.keycode[0] = 0x53;
			buffstate=4;
			bufflock=1;
			return 1;
		}
		return 0;
	}	
	if(buffTextState==0 && buffstate==0){
		_delay_ms(1);
//		LED_state_save=LED_state;
//				if(LED_state & 0b00000001){
//					PORTA |= 0b00010000;
//				}
		if (!(LED_state & 0b00000001)){
			LED_state_save=LED_state;
			kbdBuffer.modifier = 0;
			kbdBuffer.keycode[0] = 0x53;
			buffstate=4;
			bufflock=1;
			return 1;
		}
	}
	if(buffstate==0){
		ch=buffText[buffTextState];
		if(ch >= '0' && ch <= '9') {
			kbdBuffer.modifier = 0;
			kbdBuffer.keycode[0] = (ch == '0') ? 39 : 30+(ch-'1');
		} else if(ch==13){
			kbdBuffer.modifier = 0;
			kbdBuffer.keycode[0] = 0x28;
		} else if(ch==' '){
			kbdBuffer.modifier = 0;
			kbdBuffer.keycode[0] = 0x2c;
		} else {
			i=ch % 10;
			buffKey[2]=(i==0 ? 0x62 : i + 0x58);
			ch/=10;
			i=ch % 10;
			buffKey[1]=(i==0 ? 0x62 : i + 0x58);
			i=ch / 10;
			buffKey[0]=(i==0 ? 0x62 : i + 0x58);
			buildkey();
		}
		buffTextState++;
	}else{
		buildkey();
	}
	return 1;
}

/* The following function advances sin/cos by a fixed angle
 * and stores the difference to the previous coordinates in the report
 * descriptor.
 * The algorithm is the simulation of a second order differential equation.
 */
static void advanceCircleByFixedAngle(void)
{
char    d;

#define DIVIDE_BY_64(val)  (val + (val > 0 ? 32 : -32)) >> 6    /* rounding divide */
    reportBuffer.dx = d = DIVIDE_BY_64(cosinus);
    sinus += d;
    reportBuffer.dy = d = DIVIDE_BY_64(sinus);
    cosinus -= d;
  //reportBuffer.dx=10;
  //reportBuffer.dy=10;

}

/* ------------------------------------------------------------------------- */
uchar usbFunctionWrite(uint8_t * data, uchar len) {
	//PORTA |= 0b00010000;
	LED_state=data[1];
	if (LED_state & 0b00000001){
		// data[1] & 0b00000001 - NUM LOCK
		// data[1] & 0b00000010 - CAPS LOCK
		// data[1] & 0b00000100 - SCROLL LOCK
		PORTA |= 0b00010000;		
	}else{
		PORTA &= ~ 0b00010000;
	}
	return 1; // Data read, not expecting more
}

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    /* The following requests are never used. But since they are required by
     * the specification, we implement them in this example.
     */
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
//        DBG1(0x50, &rq->bRequest, 1);   /* debug output: print our request */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* we only have one report type, so don't look at wValue */
            usbMsgPtr = (void *)&reportBuffer;
            return sizeof(reportBuffer);
		}else if(rq->bRequest == USBRQ_HID_SET_REPORT){ // if wLength == 1, should be LED state
			/*if(rq->wLength.word == 1){
				PORTA |= 0b00010000;
			}*/			
			//return (rq->wLength.word == 1) ? USB_NO_MSG : 0;
			return USB_NO_MSG;
        }else if(rq->bRequest == USBRQ_HID_GET_IDLE){
            usbMsgPtr = &idleRate;
            return 1;
        }else if(rq->bRequest == USBRQ_HID_SET_IDLE){
            idleRate = rq->wValue.bytes[1];
        }
    }else{
        /* no vendor specific requests implemented */
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */



int main(void)
{
uchar   i;
uchar button1=1;
uchar button1on=0;
uchar button2=1;
uchar button2on=0;
uchar button3=0;
uchar button3on=1;
//uchar kbddown=0;
DDRD |= 0b01000000;
DDRA |= 0b00011000;
PORTA |= 0b00001111;
   // wdt_enable(WDTO_1S);
    /* Even if you don't use the watchdog, turn it off here. On newer devices,
     * the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
     */
    /* RESET status: all port bits are inputs without pull-up.
     * That's the way we need D+ and D-. Therefore we don't need any
     * additional hardware initialization.
     */
  //  odDebugInit();
  //  DBG1(0x00, 0, 0);       /* debug output: main starts */
    reportBuffer.id=0x02;
	kbdBuffer.id=0x01;
	
	_delay_ms(20);
	
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    i = 0;
    while(--i){             /* fake USB disconnect for > 250 ms */
      //  wdt_reset();
        _delay_ms(1);
    }
    usbDeviceConnect();
	
    sei();
    //DBG1(0x01, 0, 0);       /* debug output: main loop starts */
    for(;;){                /* main event loop */
        //DBG1(0x02, 0, 0);   /* debug output: main loop iterates */
        //wdt_reset();
        usbPoll();//
		_delay_ms(1);
	//for(int jk=0;jk<40000;jk++){
		if(usbInterruptIsReady()){
			testpoll=0;	
		}		
        if(usbInterruptIsReady() && button1on){
			PORTD |= 0b01000000;
			
            /* called after every poll of the interrupt endpoint */
            advanceCircleByFixedAngle();
            //DBG1(0x03, 0, 0);   /* debug output: interrupt report prepared */
            usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
        }
	//}	
		_delay_ms(1);
		PORTD &= ~ 0b01000000;
		_delay_ms(1);
		
		if(usbInterruptIsReady() && (button2on || buffstate)){
			PORTD |= 0b01000000;
			
			/* called after every poll of the interrupt endpoint */
			//buildkey();			
			if(!kbdPrintText()) {
				button2on=0;
				buffTextState=0;
			}				
			//DBG1(0x03, 0, 0);   /* debug output: interrupt report prepared */
			usbSetInterrupt((void *)&kbdBuffer, sizeof(kbdBuffer));
		}

		_delay_ms(1);
		PORTD &= ~ 0b01000000;
		_delay_ms(1);
		
		if(PINA & 0b00000001 && button1==0){
			button1=1; //Кнопку отпустили
			if(button1on){
				button1on=0;		
			}else{
				button1on=1;		
			}
		}else if(!(PINA & 0b00000001) && button1)	{
			button1=0; //Кнопку нажали, сбросили флаг 1 раз
		}	
		if(PINA & 0b00000100 && button2==0){
			button2=1; //Кнопку отпустили
			if(button2on){
				button2on=0;
			}else{
				button2on=1;
			}
		}else if(!(PINA & 0b00000100) && button2)	{
			button2=0; //Кнопку нажали, сбросили флаг 1 раз
		}
		
		if(PINA & 0b00000010 && button3==0){
			button3=1;
			if(button3on){
				button3on=0;
				PORTA &= ~ 0b00001000;
			}else{
				button3on=1;
				PORTA |= 0b00001000;
			}
		}else if(!(PINA & 0b00000010) && button3)	{
			button3=0;
		}	
		testpoll++;
		if(testpoll>3000){
			PORTA |= 0b00010000;
			cli();
			usbDeviceDisconnect();
			_delay_ms(100);
			usbDeviceConnect();
			sei();
			testpoll=0;
			//PORTA &= ~ 0b00010000;
		}				
    }
	return 0;
}

/* ------------------------------------------------------------------------- */ 